@@ -24,7 +24,7 @@ from .stack import Stack
 from .spec import AbsSpec, LetSpec, MapSpec, CondSpec, PythonSpec, SeqSpec, RetSpec, TopSpec, AbsValue, NameValue, DataValue, ret_prefix_to_str, get_task_depends_on, sort_tasks, get_dep_set_spec, get_python_task_non_dependency_params, get_task_depends_on, preproc_tasks, get_python_task_dependency_params
 import jsonpickle
 from tx.readable_log import format_message, getLogger
-from typing import List, Any, Dict, Tuple, Set, Callable, TypeVar
+from typing import List, Any, Dict, Tuple, Set, Callable, TypeVar, ClassVar
 from dataclasses import dataclass
 from abc import ABC, abstractmethod
 
@@ -48,14 +48,24 @@ class TaskId(AbsTask):
     
 @dataclass
 class BaseTask(TaskId):
+    log_error: ClassVar[bool] = False
+    
     def run(self, results: Dict[str, Any], subnode_results: Dict[str, Any], queue: DependentQueue) -> Tuple[Dict[str, Any], Dict[str, Either]]:
         logger.debug("BaseTask.run: restuls = %s", results)
-        return mbind(self.baseRun, results, subnode_results, queue)
+        try:
+            return mbind(self.baseRun, results, subnode_results, queue, log_error=self.log_error)
+        except Exception as e:
+            err = (str(e), traceback.format_exc())
+            logger.error(str(err))
+            queue.put_output({":error:": Right(err)})
+            queue.close()
+            raise
+            
 
     @abstractmethod
     def baseRun(self, results: Dict[str, Any], subnode_results: Dict[str, Any], queue: DependentQueue) -> Tuple[Dict[str, Any], Dict[str, Either]]:
         pass
-    
+
 
 @dataclass
 class Task(BaseTask):
@@ -79,7 +89,9 @@ class Task(BaseTask):
             if not isinstance(result, Either):
                 result = Right(result)
         except Exception as e:
-            result = Left((str(e), traceback.format_exc()))
+            err = (str(e), traceback.format_exc())
+            logger.error(str(err))
+            result = Left(err)
         return {}, Right({self.name: result})
 
 
@@ -97,6 +109,7 @@ class DynamicMap(BaseTask):
     subnode_env: Dict[str, str]
     ret_prefix: List[Any]
     level: int
+    log_error: ClassVar[bool] = True
 
     def baseRun(self, results: Dict[str, Any], subnode_results: Dict[str, Any], queue: DependentQueue) -> Tuple[Dict[str, Any], Dict[str, Either]]:
         hold_id = queue.put(Hold(), is_hold=True)
@@ -115,6 +128,7 @@ class DynamicGuard(BaseTask):
     data: Dict[str, Any]
     subnode_top: Dict[str, str]
     ret_prefix: List[Any]
+    log_error: ClassVar[bool] = True
     
     def baseRun(self, results: Dict[str, Any], subnode_results: Dict[str, Any], queue: DependentQueue) -> Tuple[Dict[str, Any], Dict[str, Either]]:
         logger.debug("DynamicCond.baseRun: before hold")
@@ -153,7 +167,7 @@ class Seq(BaseTask):
     task_id: str
 
     def baseRun(self, results: Dict[str, Any], subnode_results: Dict[str, Any], queue: DependentQueue) -> Tuple[Dict[str, Any], Dict[str, Either]]:
-        data = {**{k:Right(v) for k,v in self.data.items()}, **{name: results[name] for name in self.depends_on}}
+        data = {**{k:Right(v) for k,v in self.data.items()}, **{name: Right(results[name]) for name in self.depends_on}}
         return execute(self.spec, data, self.ret_prefix)
 
     
@@ -228,7 +242,7 @@ def execute(spec: AbsSpec, data: Dict[str, Either], ret_prefix: List[Any]) -> Tu
             ret.update(sub_ret)
             data = {**data, **sub_result.value}
             result = {**result, **sub_result.value}
-        return ret, result
+        return ret, Right(result)
     elif isinstance(spec, PythonSpec):
         try:
             mod = import_module(spec.mod) if spec.mod != "" else builtins
@@ -240,10 +254,10 @@ def execute(spec: AbsSpec, data: Dict[str, Either], ret_prefix: List[Any]) -> Tu
                 return {}, errors[0]
             args2 = {k: v.value for k, v in args0.items()}
             args, kwargs = split_args(args2)
-            logger.debug(format_message("execute", "PythonSpec", {}))
+            logger.debug(format_message("execute", "PythonSpec before", {"spec.name": spec.name, "spec.mod": spec.mod, "func": lambda: func, "spec.func" : spec.func, "spec.params": spec.params, "args": args, "kwargs": kwargs}))
             
             result = func(*map(lambda x: x[1], sorted(args.items(), key=lambda x: x[0])), **kwargs)
-            logger.debug(format_message("execute", "PythonSpec", {"spec.name": spec.name, "spec.mod": spec.mod, "func": lambda: func, "spec.func" : spec.func, "spec.params": spec.params, "args": args, "kwargs": kwargs, "result": result, "func(1)": func(1)}))
+            logger.debug(format_message("execute", "PythonSpec after", {"spec.name": spec.name, "spec.mod": spec.mod, "func": lambda: func, "spec.func" : spec.func, "spec.params": spec.params, "args": args, "kwargs": kwargs, "result": result}))
             if not isinstance(result, Either):
                 result = Right(result)
         except Exception as e:
@@ -258,13 +272,15 @@ def execute(spec: AbsSpec, data: Dict[str, Either], ret_prefix: List[Any]) -> Tu
         raise RuntimeError(f'unsupported spec type {spec}')
     
     
-def mbind(job_run : Callable[[Dict[str, Any], Dict[str, Any], DependentQueue], Either], params: Dict[str, Either], subnode_results: Dict[str, Either], queue: DependentQueue) -> Tuple[Any, Dict[str, Any]]:
+def mbind(job_run : Callable[[Dict[str, Any], Dict[str, Any], DependentQueue], Either], params: Dict[str, Either], subnode_results: Dict[str, Either], queue: DependentQueue, log_error: bool) -> Tuple[Any, Dict[str, Any]]:
     resultv = {}
     subnode_resultv = {}
     for k, v in params.items():
         if isinstance(v, Left):
             resultj = v
             ret: Dict[str, Any] = {}
+            if log_error:
+                queue.put_output({":error:": Right(v.value)})
             break
         else:
             resultv[k] = v.value
@@ -273,6 +289,8 @@ def mbind(job_run : Callable[[Dict[str, Any], Dict[str, Any], DependentQueue], E
             if isinstance(v, Left):
                 resultj = v
                 ret = {}
+                if log_error:
+                    queue.put_output({":error:": Right(v.value)})
                 break
             else:
                 subnode_resultv[k] = v.value
@@ -395,7 +413,7 @@ def generate_tasks(queue: DependentQueue, spec: AbsSpec, data: Dict[str, Any], e
             generate_tasks(queue, sub, data, env=sub_env, ret_prefix=ret_prefix + [task_name(i, sub, "top")], hold=hold, level=level)
     elif isinstance(spec, SeqSpec):
         dependencies_dict = get_task_depends_on_dict(env, spec) 
-        task = Seq(spec=spec, data=data, depends_on=set(dependencies_dict.keys()), task_id=ret_prefix_to_str(ret_prefix, False), ret_prefix=ret_prefix)
+        task = Seq(spec=spec, data=data, depends_on=set(dependencies_dict.keys()), task_id=ret_prefix_to_str(ret_prefix, False), ret_prefix=ret_prefix + ["@seq"])
         enqueue_task(queue, task, {**inverse_dict(dependencies_dict), **{name: set() for name in hold}}, {})
     elif isinstance(spec, PythonSpec):
         name = spec.name
